package de.hft.stuttgart.citydoctor2.connect.edge;

import de.hft.stuttgart.citydoctor2.connect.edge.CppFeature;
import de.hft.stuttgart.citydoctor2.connect.edge.CppReferenceHandlingBase;
import de.hft.stuttgart.citydoctor2.connect.edge.NativeException;
import de.hft.stuttgart.citydoctor2.connect.edge.NativePointerCastException;

/**
 * 
 * @author PiR
 * @version 1.0
 *
 */
public class AutoPro extends CppReferenceHandlingBase
{
    
    // ----- native methods
    // create and release stored C++ polygon check object
    private native long createCppAutoPro() 
        throws OutOfMemoryError;
    private native void disposeCppAutoPro(long ptrToCppAutoPro) 
        throws NativePointerCastException;
    
    // native methods
 
    private native boolean addFeature(long ptrToCppAutoPro, long ptrToCppFeature) 
            throws NativeException, IllegalArgumentException;    

    private native boolean setSettingsExport(long ptrToCppAutoPro, long ptrToCppSettings) 
            throws NativeException, IllegalArgumentException; 

    private native boolean setSettingsGeneral(long ptrToCppAutoPro, long ptrToCppSettings) 
            throws NativeException, IllegalArgumentException;     

    private native boolean setSettingsBool(long ptrToCppAutoPro, long ptrToCppSettings) 
            throws NativeException, IllegalArgumentException;        
    
    private native boolean setSettingsMerge(long ptrToCppAutoPro, long ptrToCppSettings) 
            throws NativeException, IllegalArgumentException;       
    
    private native boolean setSettingsAdvancedSweep(long ptrToCppAutoPro, long ptrToCppSettings) 
            throws NativeException, IllegalArgumentException;    
    
    private native boolean applyAutoProMethods(long ptrToCppAutoPro) 
        throws NativeException, IllegalArgumentException;

    public native long getHealResult(long ptrToCppAutoPro, long ptrToCppFeature)
            throws NativeException, IllegalArgumentException;
    
    /**
     * Constructor that gets a VPDFeature. All check method will be invoked on this feature. 
     * Remember to use the createCppObject method 
     * @param feature All check methods will use the data of this feature
     *  
     * @see {@link createCppObject} {@link disposeCppObject}
     */
    public AutoPro()
    {
    }    
    
        
    /**
     * Creates an C++ object to use the native methods.
     * 
     * @throws IllegalStateException
     *             Thrown if the encapsulated "long-Pointer" isn't 0
     * @throws NativeOutOfMemoryException
     *             If there is no more memory on the native side
     * 
     * @see {@link disposeCppObject}
     * @see de.hft.stuttgart.citydoctor.connect.edGe.CppReferenceHandlingBase#createCppObject()
     */
    public void createCppObject() 
            throws OutOfMemoryError, IllegalStateException
    {
        if ( 0L != ptrToCppObject)
        {
            throw new IllegalStateException("Can't create new C++ object, there is allready one!");
        }
        // Maybe an exception is thrown and there should be no crap in ptrToCppObject
        long tmpPointer = createCppAutoPro();
        
        ptrToCppObject = tmpPointer;
    }


    @Override
    public void disposeCppObject()
            throws NativePointerCastException
    {
        if ( 0L != ptrToCppObject)
        {
            disposeCppAutoPro(this.ptrToCppObject);
            ptrToCppObject = 0x0;
        }
    }   
    
    public void ap_addFeature(CppFeature feature)
            throws NativeException, IllegalArgumentException
    {
        checkNullPointer();
        addFeature(ptrToCppObject, feature.ptrToCppObject);
    }

    public void ap_setSettingsExport(AutoProSettingsExport settings)
            throws NativeException, IllegalArgumentException
    {
        checkNullPointer();
        setSettingsExport(ptrToCppObject, settings.ptrToCppObject);
    } 
    
    public void ap_setSettingsGeneral(AutoProSettingsGeneral settings)
            throws NativeException, IllegalArgumentException
    {
        checkNullPointer();
        setSettingsGeneral(ptrToCppObject, settings.ptrToCppObject);
    }      

    public void ap_setSettingsBool(AutoProSettingsBool settings)
            throws NativeException, IllegalArgumentException
    {
        checkNullPointer();
        setSettingsBool(ptrToCppObject, settings.ptrToCppObject);
    }       
    
    public void ap_setSettingsMerge(AutoProSettingsMerge settings)
            throws NativeException, IllegalArgumentException
    {
        checkNullPointer();
        setSettingsMerge(ptrToCppObject, settings.ptrToCppObject);
    }       
    
    public void ap_setSettingsAdvancedSweep(AutoProSettingsAdvancedSweep settings)
            throws NativeException, IllegalArgumentException
    {
        checkNullPointer();
        setSettingsAdvancedSweep(ptrToCppObject, settings.ptrToCppObject);
    }  
    
    public void ap_applyAutoProMethods()
            throws NativeException, IllegalArgumentException
    {
        checkNullPointer();
        applyAutoProMethods(ptrToCppObject);
    }
    
	public long ap_getHealResult(CppFeature feature) {
        checkNullPointer();
        return getHealResult(ptrToCppObject, feature.ptrToCppObject);
	}
    

}
